#!/bin/sh

update_meta_links()
{
    builddir="$1"

    if test -f "${builddir}/meta.txt" ; then
        url=
        url_stage1=
        url_original=
        url_stage1_original=
        metatext=
        flag_change=
        while read metaline ; do
            case "${metaline}" in
                "url_lfs-${LFS_VERSION}:"*)
                    url="${metaline#*:}"
                    flag_change=1
                ;;
                "url_lfs-${LFS_VERSION}.stage1:"*)
                    url_stage1="${metaline#*:}"
                    flag_change=1
                ;;
                "url.lfs:"*)
                    url_original="${metaline#*:}"
                ;;
                "url.lfs.stage1:"*)
                    url_stage1_original="${metaline#*:}"
                ;;
                *)
                    metatext="${metatext}${metaline}
"
                ;;
            esac
        done < "${builddir}/meta.txt"
        metatext="${metatext%
}"

        if test -n "${flag_change}" ; then
            if test -n "${url}" ; then
                metatext="${metatext}
url.lfs:${url}"
            else
                metatext="${metatext}
url.lfs:${url_original}"
            fi
            if test -n "${url_stage1}" ; then
                metatext="${metatext}
url.lfs.stage1:${url_stage1}"
            else
                metatext="${metatext}
url.lfs.stage1:${url_stage1_original}"
            fi
        metatext="${metatext#
}"
            printf "%s\n" "${metatext}" > "${builddir}/meta.txt"
        fi
    fi
}

LFS_VERSION="${1}"

if test -z "${LFS_VERSION}" ; then
    echo "First argument is version of LFS"
fi

case "${LFS_VERSION}" in
    11.*|10.*|9.*|8.*)
        :
    ;;
    --help|-h)
        echo "Usage:
${0} LFS_VERSION <workdir>
If second argument <workdir> is empty, creation of build listing is skipped.
"
    ;;
    *)
        echo "Version ${LFS_VERSION}" is unsupported.
        exit 1
    ;;
esac

major_version="${LFS_VERSION%%.*}"
mkdir -p "./${LFS_VERSION}"

main_templates="$(pwd)/main-templates"
main_target="$(pwd)/${LFS_VERSION}/build.tex"

sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${main_templates}/start" > "${main_target}"
if test ${major_version} -ge '10' ; then
    sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${main_templates}/build-cross" >> "${main_target}"
else
    sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${main_templates}/build-tools" >> "${main_target}"
fi
sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${main_templates}/final" >> "${main_target}"


# ------------------------------
WORKDIR="${2}"

if test -z "${WORKDIR}" ; then
    exit 0
fi
templates="$(pwd)/listing-templates"
target="$(pwd)/${LFS_VERSION}/listing.tex"

printf "%s\n" "Generating listing for builds, version lfs-${LFS_VERSION}:"
# TODO: name hashing

cat "${templates}/global_header" > "${target}"

sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${templates}/section_header_pass1" >> "${target}"
    (
    cd "${WORKDIR}"
    if test ${major_version} -ge '10' ; then
        printf "%s\n" "Generating pass1 builds..."
        lspkg lfs,pass1 tools-${LFS_VERSION}.src | while read name ; do
            update_meta_links "./ports/packages/${name}"
            printf "%s\n" "./ports/packages/${name}"
        done | build2markup -a 'pass1 crosstch' -B -o "${target}.pass1" > /dev/null
        # names of subsections in pass1 should not repeat names in main. Otherwise they would be rewritten.
        # skip names containing pass1, pass2 and add 'pass1' to the rest
        sed -e '/pass1-/b;/pass2-/b; s/\\subsection{/\\subsection{tmp /' "${target}.pass1" >> "${target}"

        printf "%s\n" "Generating pass1inside builds..."
        sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${templates}/section_header_pass1inner" >> "${target}"
        lspkg lfs,passinside tools-inside-${LFS_VERSION}.src | while read name ; do
            update_meta_links "./ports/packages/${name}"
            printf "%s\n" "./ports/packages/${name}"
        done | build2markup -a 'passinside' -B -o "${target}.passinside" > /dev/null
        sed -e '/pass1-/b;/pass2-/b; s/\\subsection{/\\subsection{tmp /' "${target}.passinside" >> "${target}"

        printf "%s\n" "Generating main builds..."
        sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${templates}/section_header_main_cross" >> "${target}"
        lspkg lfs lfs-part1-${LFS_VERSION}.src,lfs-part2-${LFS_VERSION}.src | while read name ; do
            update_meta_links "./ports/packages/${name}"
            printf "%s\n" "./ports/packages/${name}"
        done | build2markup -p "../files/ports/packages" -B -o "${target}.main" > /dev/null
        cat "${target}.main" >> "${target}"

        rm "${target}.pass1" "${target}.passinside" "${target}.main"
        cd "$(dirname "${target}")"
        rm tmp.*
    else
        printf "%s\n" "Generating pass1 builds..."
        lspkg lfs,pass1 tools-${LFS_VERSION}.src | while read name ; do
            update_meta_links "./ports/packages/${name}"
            printf "%s\n" "./ports/packages/${name}"
        done | build2markup -a 'pass1 tools' -B -o "${target}.pass1" > /dev/null
        # names of subsections in pass1 should not repeat names in main. Otherwise they would be rewritten.
        # skip names containing pass1, pass2 and add 'pass1' to the rest
        sed -e '/pass1-/b;/pass2-/b; s/\\subsection{/\\subsection{tmp /' "${target}.pass1" >> "${target}"

        printf "%s\n" "Generating main builds..."
        sed -e "s/%<LFS_VERSION>/${LFS_VERSION}/g" "${templates}/section_header_main_tools" >> "${target}"
        lspkg lfs lfs-part1-${LFS_VERSION}.src,lfs-part2-${LFS_VERSION}.src | while read name ; do
            update_meta_links "./ports/packages/${name}"
            printf "%s\n" "./ports/packages/${name}"
        done | build2markup -p "../files/ports/packages" -B -o "${target}.main" > /dev/null
        cat "${target}.main" >> "${target}"

        rm "${target}.pass1" "${target}.passinside" "${target}.main"
        cd "$(dirname "${target}")"
        rm tmp.*
    fi
    )

cat "${templates}/global_footer" >> "${target}"

printf "%s\n%s\n" "Result:" "${target}"
